'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = connectWpPost;

var _debug = require('../util/debug');

var _debug2 = _interopRequireDefault(_debug);

var _contentTypesManager = require('../util/content-types-manager');

var _contentTypesManager2 = _interopRequireDefault(_contentTypesManager);

var _invariants = require('../invariants');

var _invariants2 = _interopRequireDefault(_invariants);

var _base2 = require('./base');

var _base3 = _interopRequireDefault(_base2);

var _util = require('./util');

var _actions = require('../redux/actions');

var _sagas = require('../redux/sagas');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

/**
 * Connect a component to a single entity from WordPress.
 *
 * @example Built-in content type, derived slug identifier:
 * ```js
 * const { Page } from 'kasia/types'
 * connectWordPress(Page, (props) => props.params.slug)(Component)
 * ```
 *
 * @example Built-in content type, explicit ID identifier:
 * ```js
 * const { Post } from 'kasia/types'
 * connectWordPress(Post, 16)(Component)
 * ```
 *
 * @example Custom content type, derived identifier:
 * ```js
 * connectWordPress('News', (props) => props.params.slug)(Component)
 * ```
 *
 * @param {String} contentType Content type of the WP entity to fetch
 * @param {Function|String|Number} id Entity's ID/slug/a function that derives either from props
 * @param {Function} [shouldUpdate] Receives thisProps & nextProps and returns
 *                                  bool indicating whether data should be re-fetched
 * @returns {Function} Decorated component
 */
function connectWpPost(contentType, id, shouldUpdate) {
  _invariants2.default.isString('contentType', contentType);
  _invariants2.default.isIdentifierArg(id);

  if (shouldUpdate) {
    _invariants2.default.isShouldUpdate(shouldUpdate);
  }

  var typeConfig = _contentTypesManager2.default.get(contentType);

  return function (target) {
    var displayName = target.displayName || target.name;

    _invariants2.default.isNotWrapped(target, displayName);

    var KasiaConnectWpPostComponent = function (_base) {
      _inherits(KasiaConnectWpPostComponent, _base);

      function KasiaConnectWpPostComponent() {
        _classCallCheck(this, KasiaConnectWpPostComponent);

        return _possibleConstructorReturn(this, _base.apply(this, arguments));
      }

      KasiaConnectWpPostComponent.preload = function preload(props) {
        (0, _debug2.default)(displayName, 'connectWpPost preload with props:', props);
        _invariants2.default.isValidContentType(typeConfig, contentType, displayName + ' component');
        var action = (0, _actions.createPostRequest)(contentType, (0, _util.identifier)(displayName, id, props));
        return [_sagas.fetch, action];
      };

      KasiaConnectWpPostComponent.prototype._getRequestWpDataAction = function _getRequestWpDataAction(props) {
        (0, _debug2.default)(displayName, 'connectWpPost request with props:', props);
        var realId = (0, _util.identifier)(displayName, id, props);
        return (0, _actions.createPostRequest)(contentType, realId);
      };

      KasiaConnectWpPostComponent.prototype._makePropsData = function _makePropsData(props) {
        var entities = this.props.wordpress.entities[typeConfig.plural];

        if (entities) {
          var lookupId = (0, _util.identifier)(displayName, id, props);
          var idKey = void 0;

          if (typeof lookupId === 'string') {
            idKey = 'slug';
          } else {
            idKey = 'id';
          }

          for (var key in entities) {
            var entity = entities[key];
            if (entity[idKey] == lookupId) {
              // eslint-disable-line eqeqeq
              return entity;
            }
          }
        }

        return null;
      };

      KasiaConnectWpPostComponent.prototype._shouldUpdate = function _shouldUpdate() {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        if (shouldUpdate) {
          return shouldUpdate.apply(undefined, args);
        } else {
          var thisProps = args[0],
              nextProps = args[1];

          var entity = this._makePropsData(nextProps);
          var identChanged = (0, _util.identifier)(displayName, id, nextProps) !== (0, _util.identifier)(displayName, id, thisProps);
          return !entity && identChanged;
        }
      };

      KasiaConnectWpPostComponent.prototype.componentWillMount = function componentWillMount() {
        _invariants2.default.isValidContentType(typeConfig, contentType, displayName + ' component');
        _base.prototype.componentWillMount.call(this);
      };

      return KasiaConnectWpPostComponent;
    }((0, _base3.default)(target, contentType, null));

    return (0, _util.connect)(KasiaConnectWpPostComponent);
  };
}